//Click Bank - Storage and handling of the clicks in the system.
function ClickBank()
{
	this.zoom = 1;
	this.vault = {};
	this.vaultCount = 0;
	this.vaultArchive = [];
	this.vaultGroups = {};
	
	this.heldClicks = [];
	
	this.heldE = null;
	this.lastE = null;
	
	//Init - Install the click listeners
	this.init = function()
	{
		$("#gp0Canvas").click(function(e)
		{
			ClickBank.click(e);
		});
		this.zoom = 1/$("#gp0Canvas").css("zoom");
		if (!this.zoom) this.zoom = 1;

		Render.gp0Canvas.addEventListener("mousedown", function(e)
		{
			ClickBank.heldE = e;
			ClickBank.lastE = e;
		});
		Render.gp0Canvas.addEventListener("mouseup", function(e)
		{
			ClickBank.heldE = null;
		});
	};
	
	//Add a new vault entry
	//All functions will result in two params returned when clicked (clickID, vaultEntry)
	this.add = function(data)
	{
		if (data.id == undefined || data.id == "") return false;
		if (data.allowoverwrite == undefined) data.allowoverwrite = false;
		if (this.vault[data.id] != undefined && !data.allowoverwrite) return true;
		
		this.vault[data.id] = data;
		this.vaultCount++;
		
		if (data.group != undefined)
		{
			if (this.vaultGroups[data.group] == undefined) this.vaultGroups[data.group] = [];
			this.vaultGroups[data.group].push(data.id);
		}
		
		return true;
	};
	
	//Clear a vault entry
	this.clear = function(id)
	{
		var data = this.vault[id];
		if (data)
		{
			if (data && data.group)
			{
				var groupids = [];
				for(var key in this.vaultGroups[data.group])
					if (this.vaultGroups[data.group][key] != id)
						groupids.push(this.vaultGroups[data.group][key]);
				if (groupids.length > 0) this.vaultGroups[data.group] = groupids;
				else delete this.vaultGroups[data.group];
			}
			
			delete this.vault[id];
			this.vaultCount--;
		}
	};
	
	//Clear this group from vault
	this.clearGroup = function(group)
	{
		if (this.vaultGroups[group] == undefined) return false;
		
		for(var key in this.vaultGroups[group])
		{
			delete this.vault[this.vaultGroups[group][key]];
			this.vaultCount--;
		}
		
		delete this.vaultGroups[group];
	};
	
	//Clear all in the vault
	this.clearAll = function()
	{
		this.vault = {};
		this.vaultCount = 0;
		this.vaultGroups = {};
	};
	
	//Archives the current vault
	this.archive = function()
	{
		this.vaultArchive.unshift(this.vault);
		this.clearAll();
	};
	
	//Restore the latest archived vault
	this.unarchive = function()
	{
		this.vault = this.vaultArchive.shift();
		this.vaultCount = 0;
		var vault;
		for(var key in this.vault)
		{
			vault = this.vault[key];
			if (vault.group != undefined)
			{
				if (this.vaultGroups[vault.group] == undefined) this.vaultGroups[vault.group] = [];
				this.vaultGroups[vault.group].push(vault.id);
			}
			this.vaultCount++;
		}
	};
	
	//Click function
	this.click = function(e)
	{
		if (this.vaultCount == 0) return false;
		var x = e.pageX, y = e.pageY;
		x *= this.zoom; y *= this.zoom;
		//x = Render.r2(x, "x", "gp"); y = Render.r2(y, "y", "gp")
		
		var entry, hasclicked=false;
		for(var clickID in this.vault)
		{
			if (this.vault[clickID] != undefined)
			{
				entry = this.vault[clickID];
				
				if (entry.click == undefined) continue;
				if (entry.x <= x && x <= entry.x+entry.w
				 && entry.y <= y && y <= entry.y+entry.h)
				{
					entry.click(clickID, entry);
					hasclicked = true;
				}
			}
		}
	};
	
	//Click held down function
	this.clickHeld = function()
	{
		if (this.vaultCount == 0) return false;
		if (this.heldE == null) return false;
		var e = this.heldE;
		var x = e.pageX, y = e.pageY;
		x *= this.zoom; y *= this.zoom;
		
		var entry, hasclicked=false;
		for(var clickID in this.vault)
		{
			if (this.vault[clickID] != undefined)
			{
				entry = this.vault[clickID];
				
				if (entry.held == undefined) continue;
				if (entry.x <= x && x <= entry.x+entry.w
				 && entry.y <= y && y <= entry.y+entry.h)
				{
					entry.held(clickID, entry);
					hasclicked = true;
				}
			}
		}
	};
	
	//Re-triggers the click event in one frame cycle
	this.delayClickByOneFrame = function()
	{
		this.heldClicks[this.heldClicks.length] = ({
			ttl: 2,
			e: this.lastE
		});
	};
	
	//Will trigger all delay clicks
	this.triggerDelayClicks = function()
	{
		if (this.heldClicks.length == 0) return false;
		
		var clicks = [], click;
		for(var key in this.heldClicks)
		{
			click = this.heldClicks[key];
			click.ttl--;
			if (click.ttl > 0) clicks.push(click);
			else this.click(click.e);
		}
		this.heldClicks = clicks;
	};
	
	
	//Simple debug render
	this.render = function()
	{
		var cxt = Render.tvContext, gxt = Render.gp0Context;
		
		gxt.fillStyle = "purple";
		gxt.globalAlpha = 0.4;
		
		var entry;
		for(var clickID in this.vault)
		{
			if (1 && this.vault[clickID] != undefined)
			{
				entry = this.vault[clickID];
				
				gxt.fillRect(entry.x, entry.y, entry.w, entry.h);
			}
		}
		gxt.globalAlpha = 1;
	};
};
var ClickBank = new ClickBank();