//Storage and render of different particle streams
function Particles()
{
	this.canvas = null;
	this.emitters = {};
	this.particles = {};
	
	this.canAddParticles = true;
	this.emissionRate = 2;
	
	this.imgToColor = {};
	this.imgToColor.particle_grey = "#B8B9BB";
	this.imgToColor.particle_red = "#CE0632";
	this.imgToColor.particle_white = "#F1F2F7";
	this.imgToColor.particle_yellow = "#FFEB5E";
	this.imgToColor2 = {};
	this.imgToColor2.particle_grey = [184/255, 185/255, 187/255];
	this.imgToColor2.particle_red = [206/255, 6/255, 50/255];
	this.imgToColor2.particle_white = [241/255, 242/255, 247/255];
	this.imgToColor2.particle_yellow = [255/255, 235/255, 94/255];
	
	//Update function
	this.lastRender = 0;
	this.update = function(rate)
	{
		if (typeof rate == "undefined") rate = 1;
		
		var deltatime = Date.now() - this.lastRender;
		deltatime *= rate;
		
		var emitter, emissionRate, spawnRate;
		for(var ID in this.emitters)
		{
			emitter = this.emitters[ID];
			if (emitter.endtime <= 0)
			{
				delete this.emitters[ID];
				continue;
			}
			
			spawnRate = 0;
			if (typeof emitter.effect.spawnRate != "undefined")
				spawnRate = emitter.effect.spawnRate;
			
			if (emitter.lastSpawnTime == undefined || emitter.lastSpawnTime+spawnRate < Date.now())
			{
				emissionRate = this.emissionRate;
				if (typeof emitter.effect.emissionRate != "undefined")
					emissionRate = emitter.effect.emissionRate;
				if (Game.isLagging || Game.isLagging2) emissionRate = 1;
				if (typeof this.particles[ID] == "undefined")
					this.particles[ID] = [];
				//for(var i=0; i<emissionRate; i++)
				var i = emissionRate-1, j=0;
				do
				{
					//for(var j=0; j<rate; j++)
					j = rate-1;
					do
					{
						this.particles[ID][this.particles[ID].length] = {
							obj: emitter.obj.emitParticle(),
							timer: emitter.particletime,
							effect: emitter.effect
						};
					} while(j--);
				} while(i--);
				emitter.lastSpawnTime = Date.now();
			}
			
			emitter.endtime -= deltatime;
		}
		
		var particles, particle, newParticles;
		for(ID in this.particles)
		{
			newParticles = [];
			particles = this.particles[ID];
			//for(var i=0,n=particles.length; i<n; i++)
			var i = particles.length-1;
			if (i < 0) continue;
			do
			{
				particle = particles[i];
				if (particle.timer <= 0) continue;
				
				for(var j=0; j<rate; j++)
					particle.obj.move();
				particle.timer -= deltatime;

				newParticles[newParticles.length] = particle;
			} while(i--);
				
			this.particles[ID] = newParticles;
			if (newParticles.length == 0) delete this.particles[ID];
		}
		
		this.lastRender = Date.now();
	};

	//Render function - Called independently
	this.renderPos2 = {x2: 0, y2: 0};
	this.render = function()
	{
		if (!this.canAddParticles) return true;
		var cxt = Render.tvContext, gxt = Render.gp0Context;
		var cxtPrevColor, gxtPrevColor;
		
		var particle_img, particle_imgToUse, particle_width, particle_height;
		var position, gp0Cord, gp0Cord2;
		var img;
		for(ID in this.particles)
		{
			particles = this.particles[ID];
			//for(var i=0,n=particles.length; i<n; i++)
			var i = particles.length - 1;
			if (i < 0) continue;
			
			particle = particles[i];
			position = particle.obj.position;
			particle_img = particle.effect.image;//Render.images[particle.effect.image];
			particle_width = particle.effect.w; particle_height = particle.effect.h;
			particle_imgToUse = particle_img;
			if (this.imgToColor[particle_imgToUse] == undefined) particle_imgToUse = "particle_white";
			
			if (!this.canvas)
			{
				img = document.createElement( 'canvas' );
				img.width = img.height = particle_width;
				var imgCtx = img.getContext( '2d' );
				imgCtx.fillStyle = "white";//this.imgToColor[particle_imgToUse];
				imgCtx.fillRect(0,0,particle_width,particle_height);
				
				this.canvas = img;
			} else {
				img = this.canvas;
			}
			
			var PARTICLES = particles.length;
			var _mat32Buffer = new Float32Array( 6 * PARTICLES );
			var _mat32Buffer2 = new Float32Array( 6 * PARTICLES );
			var _rects    = new Float32Array( 4 * PARTICLES );
			var _alphaBuffer = new Float32Array( 4 * PARTICLES );

			do
			{
				particle = particles[i];
				position = particle.obj.position;
				
				_rects[ i * 4 + 0 ] = 0;
				_rects[ i * 4 + 1 ] = 0;
				_rects[ i * 4 + 2 ] = particle_width;
				_rects[ i * 4 + 3 ] = particle_height;
				
				// move particles ( x, y ) only
				_mat32Buffer[ i * 6 + 0 ] = 1;
				_mat32Buffer[ i * 6 + 3 ] = 1;
				_mat32Buffer[ i * 6 + 4 ] = (position.x-particle_width/2)*Render.renderTVRatio;
				_mat32Buffer[ i * 6 + 5 ] = (position.y-particle_height/2)*Render.renderTVRatio;
				
				gp0Cord = Game.calculateGPCords(position.x, position.y);
				this.renderPos2.x2 = gp0Cord.x2; this.renderPos2.y2 = gp0Cord.y2;
				_mat32Buffer2[ i * 6 + 0 ] = 1;
				_mat32Buffer2[ i * 6 + 3 ] = 1;
				_mat32Buffer2[ i * 6 + 4 ] = this.renderPos2.x2-particle_width/2;
				_mat32Buffer2[ i * 6 + 5 ] = this.renderPos2.y2-particle_height/2+GameGP0Display.baseY;
				
		        // alpha and color
		        _alphaBuffer[ i * 4 + 0] = this.imgToColor2[particle_imgToUse][0];
		        _alphaBuffer[ i * 4 + 1] = this.imgToColor2[particle_imgToUse][1];
		        _alphaBuffer[ i * 4 + 2] = this.imgToColor2[particle_imgToUse][2];
		        _alphaBuffer[ i * 4 + 3] = 0.8;
			} while(i--);

			cxt.drawImageInstanced(PARTICLES, img, _rects, _rects, _mat32Buffer, _alphaBuffer );
			cxt.setTransform(1, 0, 0, 1, 0, 0);
			
			gxt.drawImageInstanced(PARTICLES, img, _rects, _rects, _mat32Buffer2, _alphaBuffer );
			gxt.setTransform(1, 0, 0, 1, GameGP0Display.baseX, GameGP0Display.baseY);
			
			/*
			do
			{
				particle = particles[i];
				position = particle.obj.position;
				particle_img = particle.effect.image;//Render.images[particle.effect.image];
				particle_width = particle.effect.w; particle_height = particle.effect.h;
				
				gp0Cord = Game.calculateGPCords(position.x, position.y);
				this.renderPos2.x2 = gp0Cord.x2; this.renderPos2.y2 = gp0Cord.y2;
				gp0Cord2 = Game.calculateGPCords(particle_width, particle_height, true);
				
				particle_imgToUse = particle_img;
				if (this.imgToColor[particle_imgToUse] == undefined) particle_imgToUse = "particle_white";
				cxtPrevColor = cxt.fillStyle; gxtPrevColor = gxt.fillStyle;
				cxt.fillStyle = this.imgToColor[particle_imgToUse]; gxt.fillStyle = this.imgToColor[particle_imgToUse];
				cxt.fillRect((position.x-particle_width/2)*Render.renderTVRatio, (position.y-particle_height/2)*Render.renderTVRatio, particle_width*Render.renderTVRatio, particle_height*Render.renderTVRatio);
				gxt.fillRect(this.renderPos2.x2-gp0Cord2.x2/2, this.renderPos2.y2-gp0Cord2.y2/2, gp0Cord2.x2, gp0Cord2.y2);
				cxt.fillStyle = cxtPrevColor; gxt.fillStyle = gxtPrevColor;
				
				//cxt.drawImage(particle_img, position.x-particle_width/2, position.y-particle_height/2, particle_width, particle_height);
				//gxt.drawImage(particle_img, this.renderPos2.x2-gp0Cord2.x2/2, this.renderPos2.y2-gp0Cord2.y2/2, gp0Cord2.x2, gp0Cord2.y2);
			} while(i--);*/
		}
		particles = null;
	};
	
	//Delay the render
	this.delayRender = function()
	{
		this.lastRender = Date.now();
	};
	
	//Add new Emitter to lineup
	this.addEmitter = function(x, y, velocity, spread, timer, particletime, effect)
	{
		if (!this.canAddParticles) return true;
		if (typeof effect == "undefined")
		{
			effect = {
				image: "particle_grey",
				w: 4, h: 4
			};
		}
		
		var ID = Render.generateID();
		this.emitters[ID] = {
			obj: new Emitter(new VectorZ(x, y), new VectorZ(velocity, velocity), spread),
			endtime: timer,
			particletime: particletime,
			effect: effect
		};
		this.particles[ID] = [];
	};
	
	//Clear all Particles and Emitters
	this.clearAll = function()
	{
		this.particles = {};
		this.emitters = {};
	};
};
var Particles = new Particles();

//VectorZ Class
function VectorZ(x, y)
{
	this.x = x || 0;
	this.y = y || 0;
	
	//Add a vector unit to this
	this.add = function(vector)
	{
		this.x += vector.x;
		this.y += vector.y;
	};
	
	//Gets the length of the vector
	this.getMagnitude = function()
	{
		return Math.sqrt(this.x * this.x + this.y * this.y);
	};
	
	//Gets the angle accounting for the quadrant we're in
	this.getAngle = function()
	{
		return Math.atan2(this.y,this.x);
	};
	
	//Allows us to get a new vector from angle and magnitude
	this.fromAngle = function(angle, magnitude)
	{
		return new VectorZ(magnitude * Math.cos(angle), magnitude * Math.sin(angle));
	};
};

//Particle Class
function Particle(point, velocity, acceleration)
{
	this.position = point || new VectorZ(0, 0);
	this.velocity = velocity || new VectorZ(0, 0);
	this.acceleration = acceleration || new VectorZ(0, 0);
	
	//Move the particle
	this.move = function()
	{
		this.velocity.add(this.acceleration);
		this.position.add(this.velocity);
	};
};

//Emitter Class
function Emitter(point, velocity, spread)
{
	this.position = point; // VectorZ
	this.velocity = velocity; // VectorZ
	this.spread = spread || Math.PI / 32; // possible angles = velocity +/- spread
	
	this.emitParticle = function()
	{
		// Use an angle randomized over the spread so we have more of a "spray"
		var angle = this.velocity.getAngle() + this.spread - (Math.random() * this.spread * 2);
		
		// The magnitude of the emitter's velocity
		var magnitude = this.velocity.getMagnitude();
		
		// The emitter's position
		var position = new VectorZ(this.position.x, this.position.y);
		
		// New velocity based off of the calculated angle and magnitude
		var velocity = (new VectorZ()).fromAngle(angle, magnitude);
		
		// return our new Particle!
		return new Particle(position,velocity);
	};
};